import { Observable } from 'rxjs';
import { AngularFirestore, AngularFirestoreDocument, DocumentChangeType } from '@angular/fire/firestore';
import { AngularFirestoreCollection } from '@angular/fire/firestore/collection/collection';
import firebase from 'firebase';
import { FirestoreItem, FirestoreItemFullWithIndex } from './models/firestoreItem';
import { SubCollectionQuery } from './sub-collection-query';
import { SubCollectionWriter } from './sub-collection-writer';
import WriteBatch = firebase.firestore.WriteBatch;
import DocumentData = firebase.firestore.DocumentData;
/** Used as a wrapper for adding a document, either doc or path must be specified, helps when adding multiple */
export interface AddDocumentWrapper<A> {
    /** The data to be added */
    data: A;
    /** AngularFirestoreDocument of the Document */
    docFs?: AngularFirestoreDocument;
    /** The path to the Document */
    path?: string;
}
/**
 * Action to be taken by listener if the document does not exist.
 */
export declare enum DocNotExistAction {
    /** returns a null object */
    RETURN_NULL = 0,
    /** return all the extras such as ref, path and so on but no data, kinda just ignores that the doc isn't there */
    RETURN_ALL_BUT_DATA = 1,
    /** do not return at all until it does exist */
    FILTER = 2,
    /** return doc not found error 'doc_not_found' */
    THROW_DOC_NOT_FOUND = 3
}
/** Used internally */
interface CurrentDocSubCollectionSplit {
    /** contains the document that is considered the current */
    currentDoc: FirestoreItem;
    /** sub collections of current document */
    subCollections: {
        [index: string]: any;
    };
}
/**
 * Main Class.
 *
 *
 *
 */
export declare class AngularFirestoreDeep {
    private ngFirestore;
    private defaultDocId;
    /**
     * Constructor for AngularFirestoreWrapper
     *
     * @param ngFirestore The AngularFirestore injected from an Angular Class
     * @param defaultDocId The default name of any Document when no name is given
     */
    constructor(ngFirestore: AngularFirestore, defaultDocId?: string);
    /**
     * Same as AngularFirestoreDocument.snapshotChanges but it adds the properties in FirebaseDbItem
     * and also allows for to choose action to take when document does not exist
     *
     * Important to understand this is will trigger for every change/update on the document we are listening to.
     *
     * @param docFs AngularFirestoreDocument that will be listend to
     * @param actionIfNotExist Action to take if document does not exist
     */
    listenForDoc$<A extends FirestoreItem>(docFs: AngularFirestoreDocument<FirestoreItem>, actionIfNotExist?: DocNotExistAction): Observable<A>;
    /**
     * Same as AngularFirestoreCollection.snapshotChanges but it adds the properties in FirebaseDbItem.
     *
     * Important to understand this is will trigger for every change/update on any of the documents we are listening to.
     * That means that if any document we are listening to is changed the entire object will be triggered containing the updated data.
     *
     *
     *    Example usage.
     *
     *    ngFirestoreDeep: AngularFirestoreDeep;  //  AngularFirestoreDeep variable
     *    restaurantCollectionFs = this.ngFireStore.collection('restaurants'); // AngularFirestoreCollectionRef to restaurants
     *
     *    constructor(private ngFireStore: AngularFirestore) {
     *        this.ngFirestoreDeep = new AngularFirestoreDeep(ngFireStore);  //  initialize AngularFireStoreDeep with AngularFirestore
     *    }
     *
     *    listenForRestaurants$(): Observable<RestaurantItem[]> {
     *        return this.ngFirestoreDeep.listenForCollection$<RestaurantItem>(this.restaurantCollectionFs);
     *    }
     *
     *    If you do not wish to listen for changes and only care about getting the values once
     *
     *    getRestaurants$(): Observable<RestaurantItem[]> {
     *        return this.ngFirestoreDeep.listenForCollection$<RestaurantItem>(this.restaurantCollectionFs).pipe(
     *          take(1)
     *        );
     *    }
     *
     *
     * @param collectionFs the AngularFirestoreCollection which will be listened to
     * @param documentChangeTypes list of DocumentChangeType that will be listened to, if null listen to all
     */
    listenForCollection$<A extends FirestoreItem>(collectionFs: AngularFirestoreCollection<FirestoreItem>, documentChangeTypes?: DocumentChangeType[]): Observable<A[]>;
    /**
     *
     * Allows for listening to documents and collections n deep up to the firestore max of 100 levels.
     *
     * Triggers for any change in any document that is listened to.
     *
     *
     * E.x:
     *      const subCollectionQueries: SubCollectionQuery[] = [
     *         { name: 'data' },
     *         { name: 'secure' },
     *         { name: 'variants' },
     *         { name: 'images',
     *           queryFn: ref => ref.orderBy('index'),
     *           collectionWithNames: [
     *             { name: 'secure'}
     *           ]
     *         },
     *     ];
     *
     *     this.listenForDocAndSubCollections<Product>(docFs, collections)
     *
     * Wrapper for listenForDocDeepRecursiveHelper$ so that we can cast the return to the correct type
     * All logic is in listenForDocDeepRecursiveHelper$.
     *
     * @param docFs - a docFs with potential queryFn
     * @param subCollectionQueries - see example
     */
    listenForDocDeep$<A extends FirestoreItem>(docFs: AngularFirestoreDocument, subCollectionQueries?: SubCollectionQuery[]): Observable<A>;
    /**
     * DO NOT CALL THIS METHOD, meant to be used solely by listenForDocAndSubCollections$
     */
    protected listenForDocDeepRecursiveHelper$<A extends FirestoreItem>(docFs: AngularFirestoreDocument, subCollectionQueries?: SubCollectionQuery[], actionIfNotExist?: DocNotExistAction): Observable<FirestoreItem>;
    /**
     * Listens for collections inside collections with the same name to an unlimited depth and returns all of it as an array.
     */
    listenForCollectionRecursively$<A extends FirestoreItem>(path: string, collectionKey: string, orderKey?: string): Observable<any>;
    /**
     * A replacement/extension to the AngularFirestoreCollection.add.
     * Does the same as AngularFirestoreCollection.add but can also add createdDate and modifiedDate and returns
     * the data with the added properties in FirebaseDbItem
     *
     * Used internally
     *
     * @param data the data to be added to the document, cannot contain types firestore won't allow
     * @param collectionFs the AngularFirestoreCollection where the document should be added
     * @param isAddDates if true adds modifiedDate and createdDate to the data
     * @param id if given the added document will be given this id, otherwise a random unique id will be used.
     */
    protected addSimple$<A extends DocumentData>(data: A, collectionFs: AngularFirestoreCollection, isAddDates?: boolean, id?: string): Observable<A>;
    /**
     * Add document to firestore and split it up into sub collection.
     *
     * @param data the data to be saved
     * @param collectionFs AngularFirestoreCollection reference to where on firestore the item should be saved
     * @param subCollectionWriters see documentation for SubCollectionWriter for more details on how these are used
     * @param isAddDates if true 'createdDate' and 'modifiedDate' is added to the data
     * @param docId If a docId is given it will use that specific id when saving the doc, if no docId is given a ranom id will be used.
     */
    addDeep$<A extends FirestoreItem>(data: A, collectionFs: AngularFirestoreCollection, subCollectionWriters?: SubCollectionWriter[], isAddDates?: boolean, docId?: string): Observable<A>;
    /**
     * clean FirestoreBaseItem properties from the data.
     * Usually done if you wish to save the data to firestore, since some FirestoreBaseItem properties are of non allowed types.
     *
     * Goes through each level and removes DbItemExtras
     * In case you wish to save the data
     *
     * @param data data to be cleaned
     * @param subCollectionWriters if the document has child documents the SubCollectionWriters are needed to locate them
     */
    cleanExtrasFromData<A extends FirestoreItem>(data: A, subCollectionWriters?: SubCollectionWriter[]): A;
    /**
     * Recursive method to clean FirestoreBaseItem properties from the dbItem
     *
     * @param dbItem the data to be cleaned
     * @param subCollectionWriters list of SubCollectionWriters to handle sub collections
     */
    protected removeDataExtrasRecursiveHelper<A extends FirestoreItem>(dbItem: A, subCollectionWriters?: SubCollectionWriter[]): A;
    /**
     * Firestore doesn't allow you do change the name or move a doc directly so you will have to create a new doc under the new name
     * and then delete the old doc.
     * returns the new doc once the delete is done.
     *
     * @param docFs AngularFirestoreDocument to have its id changed
     * @param newId the new id
     * @param subCollectionQueries if the document has child documents the subCollectionQueries are needed to locate them
     * @param subCollectionWriters if the document has child documents the SubCollectionWriters are needed to add them back
     */
    changeDocId$<A extends FirestoreItem>(docFs: AngularFirestoreDocument, newId: string, subCollectionQueries?: SubCollectionQuery[], subCollectionWriters?: SubCollectionWriter[]): Observable<any>;
    /**
     * Update document and child documents
     *
     * Be careful when updating a document of any kind since we allow partial data there cannot be any type checking prior to update
     * so its possible to introduce spelling mistakes on attributes and so forth
     *
     * @param data the data that is to be added or updated { [field: string]: any }
     * @param docFs AngularFirestoreDocument to be updated
     * @param subCollectionWriters if the data contains properties that should be placed in child collections and documents specify that here
     * @param isAddModifiedDate if true the modifiedDate property is added/updated on the affected documents
     */
    updateDeep$<A extends DocumentData>(data: A, docFs: AngularFirestoreDocument, subCollectionWriters?: SubCollectionWriter[], isAddModifiedDate?: boolean): Observable<any>;
    /**
     * DO NOT CALL THIS METHOD, used by update deep
     */
    protected updateDeepToBatchHelper<A>(data: A, docFs: AngularFirestoreDocument, subCollectionWriters?: SubCollectionWriter[], isAddModifiedDate?: boolean, batch?: WriteBatch): WriteBatch;
    /**
     * Used internally for updates that doesn't affect child documents
     */
    protected updateSimple$<A>(data: A, docFs: AngularFirestoreDocument, isAddModifiedDate?: boolean): Observable<boolean>;
    /**
     * Update the firestore document by path
     * Convenience method in case we do not have direct access to the AngularFirestoreDocument reference
     *
     * @param data the data that is to be added or updated { [field: string]: any }
     * @param path A string representing the path of the referenced document (relative to the root of the database).
     * @param subCollectionWriters if the data contains properties that should be placed in child collections and documents specify that here
     * @param isAddModifiedDate if true the modifiedDate property is added/updated on the affected documents
     */
    updateDeepByPath$<A>(data: A, path: string, subCollectionWriters?: SubCollectionWriter[], isAddModifiedDate?: boolean): Observable<any>;
    /**
     * Update/ add data to the firestore documents
     *
     * @param docsFs list of AngularFirestoreDocument to be have their data updated
     * @param data data to add/update
     * @param isAddModifiedDate if true the modifiedDate is added/updated
     */
    updateMultiple$<A>(docsFs: AngularFirestoreDocument[], data: A, isAddModifiedDate?: boolean): Observable<any>;
    /**
     * Update the firestore documents by paths
     *
     * @param path list of paths of documents to be have their data updated
     * @param data data to add/update
     * @param isAddModifiedDate if true the modifiedDate is added/updated
     */
    updateMultipleByPaths$<A>(paths: string[], data: A, isAddModifiedDate?: boolean): Observable<any>;
    moveItemInArray$<A extends FirestoreItemFullWithIndex>(array: A[], fromIndex: number, toIndex: number): Observable<any>;
    getBatchFromMoveItemInIndexedDocs<A extends FirestoreItemFullWithIndex>(array: Array<A>, fromIndex: number, toIndex: number, useCopy?: boolean): firebase.firestore.WriteBatch;
    protected getBatchFromTransferItemInIndexedDocs<A extends FirestoreItemFullWithIndex>(previousArray: A[], currentArray: A[], previousIndex: number, currentIndex: number, additionalDataUpdateOnMovedItem?: {
        [key: string]: any;
    }, useCopy?: boolean): firebase.firestore.WriteBatch;
    getBatchFromDeleteItemInIndexedDocs<A extends FirestoreItemFullWithIndex>(array: A[]): firebase.firestore.WriteBatch;
    protected updateIndexAfterDeleteInIndexedDocs<A extends FirestoreItemFullWithIndex>(array: A[]): Observable<any>;
    /**
     * Delete document
     * Used internally
     *
     * @param docFs AngularFirestoreDocument that is to be deleted
     */
    protected deleteSimple$(docFs: AngularFirestoreDocument): Observable<any>;
    /**
     * Delete Document and child documents
     *
     * @param docFs AngularFirestoreDocument that is to be deleted
     * @param subCollectionQueries if the document has child documents the subCollectionQueries are needed to locate them
     */
    deleteDeep$(docFs: AngularFirestoreDocument, subCollectionQueries?: SubCollectionQuery[]): Observable<any>;
    /**
     * Delete Documents
     *
     * @param docsFs - A list of AngularFireDocument that are to be deleted
     */
    deleteMultiple$(docsFs: AngularFirestoreDocument[]): Observable<any>;
    deleteMultipleByPaths$(paths: string[]): Observable<any>;
    /**
     * Delete Documents and child documents
     *
     * @param docsFs - A list of AngularFireDocument that are to be deleted
     * @param subCollectionQueries if the document has child documents the subCollectionQueries are needed to locate them
     */
    deleteMultipleDeep$(docsFs: AngularFirestoreDocument[], subCollectionQueries?: SubCollectionQuery[]): Observable<any>;
    /**
     * Delete firestore document by path
     * Convenience method in case we do not have direct access to the AngularFirestoreDocument reference
     *
     * @param path A string representing the path of the referenced document (relative to the root of the database).
     * @param subCollectionQueries if the document has child documents the subCollectionQueries are needed to locate them
     */
    deleteDocByPath$(path: string, subCollectionQueries?: SubCollectionQuery[]): Observable<any>;
    /**
     * Delete document by FirestoreItem
     *
     * Convenience method that works the same as deleteDeep$ but takes a FirestoreItem to be deleted
     *
     * @param item FirestoreItem to be deleted
     * @param subCollectionQueries if the document has child documents the subCollectionQueries are needed to locate them
     */
    deleteDeepByItem$(item: FirestoreItem, subCollectionQueries?: SubCollectionQuery[]): Observable<any>;
    /**
     * Returns an Observable containing a list of AngularFirestoreDocument found under the given docFs using the SubCollectionQuery[]
     * Mainly used to delete a docFs and its sub docs
     * @param docFs: AngularFirestoreDocument
     * @param subCollectionQueries: SubCollectionQuery[]
     */
    protected getFirestoreDocumentsDeep$<A extends FirestoreItem>(docFs: AngularFirestoreDocument, subCollectionQueries?: SubCollectionQuery[]): Observable<AngularFirestoreDocument<A>[]>;
    /**
     * Used by deleteDeepByItem$ to get all the AngularFirestoreDocuments to be deleted
     * including child documents using SubCollectionQueries
     *
     * Internal use
     * @param dbItem FirestoreItem from where we get the AngularFirestoreDocuments
     * @param subCollectionQueries if the dbItem has child documents the subCollectionQueries are needed to locate them
     */
    protected getAngularFirestoreDocumentsFromDbItem<A extends FirestoreItem>(dbItem: FirestoreItem, subCollectionQueries?: SubCollectionQuery[]): AngularFirestoreDocument[];
    /**
     * DO NOT CALL THIS METHOD, its meant as a support method for getDocs$
     */
    protected getPathsFromDbItemDeepRecursiveHelper<A extends FirestoreItem>(dbItem: FirestoreItem, subCollectionQueries?: SubCollectionQuery[]): string[];
    /**
     * DO  NOT  CALL THIS METHOD, used in addDeep and updateDeep to split the data into currentDoc and subCollections
     * Onyl goes one sub level deep;
     */
    protected splitDataIntoCurrentDocAndSubCollections<A extends FirestoreItem>(data: A, subCollectionWriters?: SubCollectionWriter[]): CurrentDocSubCollectionSplit;
    /**
     * Turn a batch into an Observable instead of Promise.
     *
     * For some reason angularfire returns a promise on batch.commit() instead of an observable like for
     * everything else.
     *
     * This method turns it into an observable
     */
    protected batchCommit$(batch: firebase.firestore.WriteBatch): Observable<any>;
    /**
     *
     * TODO make this user defined via callback
     *
     */
    protected handleRes$<A>(result$: Observable<A>, name: string, data: {
        [key: string]: any;
    }, successMessage?: string, errorMessage?: string): Observable<A>;
}
export {};
